;+=----------------------------------------------------------------------------
; Name   : example_02.pro
; Desc   : Example program no. 2 covers the use of mrdicom, which is a
;          subclass to IDLffDicom and offers writing of DICOM3.0 files and
;          direct access of DICOM tags via tag names.
; Author : (c) Oliver Thilmann, Aug 2004
;=-----------------------------------------------------------------------------
pro example_02

	mrlib_init

		; Initialize mrDicom - see the documentation

	mrdicom_setUIDs, 'mrimg', '0.0.00.0.0000.0.0'

	; -------------------------------------------------
	;    Access of dicom tags
	; -------------------------------------------------

	file    = dialog_pickfile( path='C:\' )
	if( file eq '' ) then message, 'No file selected'
	dataSet = obj_new('mrimagesetdicom', 1)
	err     = dataSet->load( file, 0 )
	if( err eq 0 ) then x = dialog_message(['Error loading image',dataSet->errMsg()])

		; dicomData() returns a pointer to an array of pointers to the dicom objects.
		; Since there is just one, we just the very first.

	dcm = (*(dataSet->DicomData()))[0]

		; See the docu for a full list of tagnames (which are case insensitive, btw.)

	print, 'Patient name    : ', dcm->getTagValue('pat_patient_name')
	print, 'Acquisition date: ', dcm->getTagValue('id_acquisition_date')

		; etc. etc.

		; plot the image - getImg returns a pointer to the image data

	img = dcm->img()
	loadct, 0
	tvscl, *img

	; --------------------------------------------------------
	;    Copying a dicom file while anonymizing patient data
	; ---------------------------------------------------------

	dcm2 = obj_new('mrdicom')

		; copy the dicom tag values from above object

	dcm2->copy, dcm

		; erase all patient related data

	dcm2->anonymize

		; just for demonstration purpose: set a new series number

	dcm2->setTagValue, 'rel_series_number', 111

		; since we set a new series number, we must also set a new
		; unique series id - see the docs for more info

	series_uid = dcm2->generateUId('series')
	dcm2->setTagValue, 'rel_series_instance_uid', series_uid

		; The image data is not copied with the rest of the data!

	dcm2->setImg, img

		; Finally write the image - mrdicom can write both
		; 2-dim image matrices as well as rgb images

	err = dcm2->write('C:\testfile1.dcm')
	if( err eq 0 ) then x = dialog_message(['Error writing file 1',dcm2->errMsg()])

	obj_destroy, dcm2

	; --------------------------------------------------------
	;    Creating a new dicom file from scratch
	; ---------------------------------------------------------

		; this is pretty straight forward

	dcm3 = obj_new('mrdicom')

		; basic tags are automatically set
		; set some more tags:

	dcm3->setTagValue, 'pat_patient_name', 'Lieschen Mueller'

		; use image data from above and write
		; Even if no series or study number is set; unique
		; ids must be provided
		; This time we use an alternative way and set them directly
		; via writing method arguments - this will not store the ids
		; in the dicom object, though!

	dcm3->setImg, img

	series_uid = dcm3->generateUID('series')
	study_uid  = dcm3->generateUID('study')

	err = dcm3->write( 'C:\testfile2.dcm', studyUID = study_uid, seriesUID = series_uid )
	if( err eq 0 ) then x = dialog_message(['Error writing file 2',dcm3->errMsg()])

	; --------------------------------------------------------
	;    Writing a RGB image
	; ---------------------------------------------------------

		; define some rgb colors

	r = indgen(256)
	g = r
	b = 255-r

		; create a true color image - need to bytescale here because
		; image values will usually extend the byte range
		; createTrueColorImg would also accept larger values, as long as
		; the r,g,b arrays hold enough elements

	bytimg = bytscl( *img )
	rgbimg = createTrueColorImg( ptr_new(bytimg), r, g, b, /flip )

		; we use the dcm3 object for writing

	dcm3->setImg, rgbimg
	err = dcm3->write( 'C:\testfile3.dcm', studyUID = study_uid, seriesUID = series_uid )
	if( err eq 0 ) then x = dialog_message(['Error writing file 2',dcm3->errMsg()])

end
