;+=----------------------------------------------------------------------------
; Name   : example_05.pro
; Desc   : Example 5, dealing with the use of mrmultidisplay
; Author : (c) Oliver Thilmann, Aug 2003
;=-----------------------------------------------------------------------------
pro example_05

	mrlib_init

		; lazy, so we use globals

	common globals, multidisplay

		; need some data to display

	mrdicom_setUIDs, 'mrimg', '0.0.00.0.0000.0.0'
	file = dialog_pickfile( path='c:\' )

	dcm  = obj_new('mrImageSetDicom', 1)
	res  = dcm->load( file, 0, /rotate )

		; in case of errors, print error message and return

	if( res eq 0 ) then begin
		x = dialog_message(['Error loading image', dcm->errMsg()])
		return
	end

	img = (*(dcm->imgData()))[0]


		; CREATE UI - only a multidisplay with 8 times 2 images
		; The mrmultidisplay object displays image sized 256*256 in a 4 * 2 matrix
		; The screen size of the object is only 768 * 512, i.e. can only display
		; 3 * 2 images at a time. Note that scrollbars are automatically added in
		; that case.

	base    = widget_base( /column )
	base1   = widget_base( base, /row )
	base2   = widget_base( base, /row )

	multidisplay  = obj_new('mrmultidisplay', base1, 768, 512, 256, 256, 4, 2 )

		; we also add a contrast object - vertical bars for a change

	contrast = obj_new('mrcontrast', base1, xsize = 100, ysize = 512 , colbars=2, /vertical )

		; and some buttons

	b1 = widget_button( base2, value='activate some elements', event_pro='button1evt')
	b2 = widget_button( base2, value='deactivate elements'   , event_pro='button2evt')
	b3 = widget_button( base2, value='label images'          , event_pro='button3evt')
	b4 = widget_button( base2, value='remove some images'    , event_pro='button4evt')

		; realize the UI

	widget_control, base, /realize

	contrast->selfregister, multidisplay, button=3

		; now set a couple of images. The origin of the coordinate system is
		; the upper left (to be compatible with reading direction),
		; first index is the x position, the second the y position.
		; It is not visible here (as we use only one image), but note that
		; all images are displayed with the same grayscale mapping.
		; i.e. if one image has absolute values from 1 to 5000 and a second
		; image from 1 to 50, the second image will be very dark!

	for x = 0, 3 do begin
		for y = 0, 1 do begin
			multidisplay->setImg, x, y, img
		end
	end

		; and finally display the image and activate the object

	multidisplay->display
	multidisplay->activate

	xmanager, 'example_05', base, event_handler='dummy_handler', /no_block

end

; dummy handler does nothing - just intercepts events
pro dummy_handler, evt
end


; 'activate some elements' button
; activating elements means outlining the image with a green frame
pro button1evt, evt
	common globals, multidisplay
	multidisplay->frame, 0, 0, /set
	multidisplay->frame, 3, 1, /set
	multidisplay->display
end

; 'deactivate elements' button
; note the dummy arguments when calling the setInactive method (not nice, I know)
pro button2evt, evt
	common globals, multidisplay
	multidisplay->frame, /delete, /all
	multidisplay->display
end

; 'label images'
; writes a label at the image bottom. There's no possibility yet to set
; label size - will come in a future release.
pro button3evt, evt
	common globals, multidisplay
	multidisplay->label, 0, 0, 'Label 1'
	multidisplay->label, 1, 1, 'another label', color=[255,0,0]
	multidisplay->display
end

; 'remove some images'
; Shows how to take an image away from mrmultidisplay
; Deleting an image will also remove labels and deactivate the deleted
; image's framge (if it was active).
; It will also cause a recalculation of absolute minimum and maximum
; image values and thus possibly effect the contrast bar.
; The effect is not visible here as we display identical images
pro button4evt, evt
	common globals, multidisplay
	multidisplay->deleteImg, 1, 1
	multidisplay->deleteImg, 3, 1
	multidisplay->display
end
