;+=----------------------------------------------------------------------------
; Name : intersect
; RCS  : $Id: intersect.pro,v 2.0 2004/08/05 17:57:54 olivert Exp olivert $
; Type : Function
; Args : void[] array1  :  input array 1, type and number of dimensions
;        void[] array2  : of the arrays maybe anything
;        nodata   = int : value to return if no value is contained
;                         in both arrays. Default is '-1'
;        xor_flag = int : if this keyword is set, the function returns
;                         only those values with are present in either
;                         one or the other array, but not in both.
; Desc : intersect two arrays and return values that are present in
;        both arrays or (if xor_flag is set) in exactly one of the two.
; Ret  : a result array or (if empty) nodata
;        int count=count  : number of elements in the resulting array
; Code :
;     x = [0,2,3,4,6,7,9]
;     y = [3,6,10,12,20]
;
; print intersection of x and y
;
;     print,intersect(x,y)
;          3        6
;
; print xor elements
;
;     print,intersect(x,y,/xor_flag)
;          0       2       4       7       9      10      12      20
;
; print values in x that are not in y
;
;     xyu=intersect(x,y,/xor_flag) & print,intersect(x,xyu)
;          0       2       4       7       9
;
; Author:
; Jeff Hicke   12/16/92
; oth 07/11/02 'bugfix' - assuming that an intersect should not return double values.
; Licns : This program is published under the LGPL. See lgpl.txt for details
;=-----------------------------------------------------------------------------
function intersect, array1, array2, nodata=nodata,xor_flag=xor_flag, count=count

	nodata = ( n_elements(nodata) eq 0 ) ? -1 : nodata

	count = 0
	if keyword_set(xor_flag) then begin
		arr1 = array1(sort(array1))
		arr2 = array2(sort(array2))
		arr1 = arr1(uniq(arr1))
		arr2 = arr2(uniq(arr2))
		array = [arr1, arr2]
		array = array(sort(array))
		idx = n_elements(array)-1
		samp=intarr(idx+1)
	 	for i = 0,idx-1 do begin
	  		if( array[i] ne array[i+1] ) then continue
	  		samp[i]   = 1
	  		samp[i+1] = 1
	  		for j=i+2, idx-1 do begin
	  			if( array[j] ne array[i] ) then break
	  			samp[j]=1
	  			if( j ge idx ) then break
	  		endfor
	  		i=j-1
	  	endfor
	  	indices=where(samp eq 0, count)
	  	if( count gt 0 ) then return, array(indices) else return, nodata
	endif else begin
		if( n_elements(array1) eq 0 or n_elements(array2) eq 0 ) then return, nodata
		array = [array1, array2]
		array = array(sort(array))
		indices = where(array eq shift( array, -1) , count)
		if( count gt 0 ) then begin
			array=array(indices)
			return, array(uniq(array))
		endif
		return, nodata
	endelse
end
